--[[
Version 0.1.0
	Initial skeleton framework
	
Version 0.2.0
	Fixed options
	
Version 0.3.0
	Added Changes Frame to display and enter changes
	
Version 0.4.0
	Moved fixed values into separate file
	Added command to display changes
	Fixed popup to work when loot looted
	Added option to charge double dkp for items
	Added option to add dkp
	Added option to subtract dkp
	
Version 0.4.1
	Added Tier 7 tokens for normal & heroic
	Fixed raidonly bug on line 124 
	Fixed issue setting min & max quality
	Added restriction to only sort table if more than 1 player hopefuly fixes single player display issue.
	
Version 0.5.0
	Added Druids to polearms to reflect with new skill they get in patch 3.0.8 
	Added clear all ticks button
	Added option to output to OFFICER chat
	Changed DoubleCost to be +50% for 10 mans and +100% for 25s 
	Added fix for accented characters in player names
	
Version 0.5.1
	Added separate return value for cost to player separate from value to raid
	Added readme.txt file to explain how to use the addon
	Added /edkp date function to display date in download file
	
Version 0.5.2
	Fix output to self option
	
Version 0.5.3
	Fix output if website has variations of class names
	Fix output if to Guild or Officer chat

Version 0.5.4
	change value in 25s to match cost
	
Version 0.5.5
	disable output to raid if you are not in a raid.

Version 0.5.6
	
--]]

local L = LibStub("AceLocale-3.0"):GetLocale("EasyDKP")
local AceAddon = LibStub("AceAddon-3.0")
EasyDKP = AceAddon:NewAddon("EasyDKP", "AceConsole-3.0", "AceEvent-3.0")
local REVISION = tonumber(("$Revision: 65 $"):match("%d+"))

EasyDKP.defaults = {
	profile = {
		enabled = true,
		debug = false,
		minquality = 4,
		maxquality = 6,
		min_raiders = 1,
		doublecost = true,
		chatoutput = "RAID", -- Output to raid by default
	},
}

EasyDKP.quality = {}
EasyDKP.quality[0] = { text = "Poor",      color = "Grey",   link=select(4, GetItemQualityColor(0))}
EasyDKP.quality[1] = { text = "Common",    color = "White",  link=select(4, GetItemQualityColor(1))}
EasyDKP.quality[2] = { text = "Uncommon",  color = "Green",  link=select(4, GetItemQualityColor(2))}
EasyDKP.quality[3] = { text = "Rare",      color = "Blue",   link=select(4, GetItemQualityColor(3))}
EasyDKP.quality[4] = { text = "Epic",      color = "Purple", link=select(4, GetItemQualityColor(4))}
EasyDKP.quality[5] = { text = "Legendary", color = "Orange", link=select(4, GetItemQualityColor(5))}
EasyDKP.quality[6] = { text = "Artifact",  color = "Gold",   link=select(4, GetItemQualityColor(6))}

EasyDKP.class = {}
EasyDKP.class[0] = { name = "Unknown", ticked = false, link = "|cff808080" }
EasyDKP.class[1] = { name = "Death Knight", ticked = false, link = "|cffc41f3b" }
EasyDKP.class[2] = { name = "Druid", ticked = false, link = "|cffff7d0a" }
EasyDKP.class[3] = { name = "Hunter", ticked = false, link = "|cffabd473" }
EasyDKP.class[4] = { name = "Mage", ticked = false, link = "|cff69ccf0" }
EasyDKP.class[5] = { name = "Paladin", ticked = false, link = "|cfff58cba" }
EasyDKP.class[6] = { name = "Priest", ticked = false, link = "|cffffffff" }
EasyDKP.class[7] = { name = "Rogue", ticked = false, link = "|cfffff569" }
EasyDKP.class[8] = { name = "Shaman", ticked = false, link = "|cff2459ff" }
EasyDKP.class[9] = { name = "Warlock", ticked = false, link = "|cff9482c9" }
EasyDKP.class[10] = { name = "Warrior", ticked = false, link = "|cffc79c6e" }

-----------------------------------------
-- Initialisation & Startup Routines
-----------------------------------------

function EasyDKP:OnInitialize()
	local AceConfigReg = LibStub("AceConfigRegistry-3.0")
	local AceConfigDialog = LibStub("AceConfigDialog-3.0")

	self.db = LibStub("AceDB-3.0"):New("EasyDKPDB", EasyDKP.defaults, "profile")
	LibStub("AceConfig-3.0"):RegisterOptionsTable("EasyDKP", self:GetOptions(), {"easydkp", "edkp"} )
	
	-- Add the options to blizzard frame (add them backwards so they show up in the proper order
	self.optionsFrame = AceConfigDialog:AddToBlizOptions("EasyDKP","EasyDKP")
	self.optionsFrame[L["About"]] = LibStub("LibAboutPanel").new("EasyDKP", "EasyDKP")
	self.db:RegisterDefaults(self.defaults)
	self:CreateClassFrame()
--~ 	self:CreateChangesFrame()
--~ 	self:CreateViewerFrame()
	
	local version = GetAddOnMetadata("EasyDKP","Version")
	self.version = ("EasyDKP %s (r%s)"):format(version, REVISION)
	self:Print(self.version.." Loaded.")
end

function EasyDKP:OnDisable()
    -- Called when the addon is disabled
end

function EasyDKP:OnEnable()
	self:RegisterEvent("CHAT_MSG_LOOT")
	-- reset changes if downloaded data has a different date from saved variable date
	if not self.db.profile.changes or self.db.profile.changes.date ~= dkp_info.date then
		self:InitialiseChanges()
	end
end

function EasyDKP:CHAT_MSG_LOOT(event, chattext)
	if self.db.profile.enabled and (GetNumRaidMembers() >= self.db.profile.min_raiders or GetNumPartyMembers() >= self.db.profile.min_raiders) then 
		local start = string.find(chattext, L["%sreceive"])
		if start then
			start = start - 1
			local name = string.sub(chattext, 1, start)
			if (name == L["You"]) then
				name = UnitName("player")
			end
			local itemid = self:GetItemInfoFromLink(chattext)
			-- local itemName, itemString, itemRarity, itemLevel, itemMinLevel, itemType, itemSubType = GetItemInfo(itemid) 
			local _, _, iRarity, iLevel = GetItemInfo(itemid)
			if(iRarity == nil) then
				iRarity = -1
			end
			if self.db.profile.debug then
				self:Print("decoded to itemid:"..itemid.." iRarity:"..iRarity.." for name:"..name..":")
			end
			-- 2 as raid default
			if(iRarity >= self.db.profile.minquality and iRarity <= self.db.profile.maxquality) then
				local value, cost = self:GetValue(itemid)
				if(value > 0) then				
					self:PopupQuestion(name, cost)
				end
			end
		else
			if self.db.profile.debug then
				self:Print("didn't find string in chattext")
			end
		end
	end
end

function EasyDKP:ShowOptionsFrame()
	if not self.frame:IsVisible() then
		self.frame:Show()
	end
end

----------------------------
-- DKP listing functions
----------------------------

function EasyDKP:ListOptions(raidonly)
	local playercount = 0
	local playerlist = {}
	for key,val in pairs(dkp_info.members) do
		if val.class == "" then 
			val.class = "Unknown"
		end
		if not raidonly or (raidonly and UnitInRaid(val.name)) then
			if self:IncludeClass(val.class) then
				playerlist[val.name] = { ["name"] = val.name, ["class"] = val.class, ["dkp"] = val.dkp * 1, ["rank"] = val.rank}
			end
		end
	end
	if raidonly then 
		for i = 1, GetNumRaidMembers() do
			local name, _, _, _, class = GetRaidRosterInfo(i)
			if not playerlist[name] then
				if self:IncludeClass(class) then
					playerlist[name] = { ["name"] = name, ["class"] = class, ["dkp"] = 0, ["rank"] = "Not on DKP List"}
				end
			end
		end
	end	
	local sorttable = {}
	playercount = 0
	-- have to put it in a table with a numeric index for sorting to work - allows opportunity to apply changes however
	for key, val in pairs(playerlist) do
		playercount = playercount + 1
		local playerdkp = val.dkp
		if self.db.profile.changes.count > 0 then
			for i = 1, self.db.profile.changes.count do
				if self.db.profile.changes[i].name == val.name then
					playerdkp = playerdkp + self.db.profile.changes[i].value
				end
			end
		end
		table.insert(sorttable, playercount, { ["name"] = val.name, ["class"] = val.class, ["dkp"] = playerdkp, ["rank"] = val.rank })
	end
	if playercount > 1 then -- only sort if more than one player
		table.sort(sorttable, sortDKP)
	end
	local channel = string.upper(self.db.profile.chatoutput)
	self:ShowMessage(L["Easy DKP list"], channel)
	playercount = 0
	for key,val in ipairs(sorttable) do
		playercount = playercount + 1
		--local chatText = playercount..": "..self:ClassColour(val.class)..val.name..string.rep(".",14-string.len(val.name))..format("%.2f",val.dkp).." ("..val.rank..")"
		local chatText = playercount..": "..val.name..string.rep(".",14-string.len(val.name))..format("%.2f",val.dkp).." ("..val.rank..")"
		self:ShowMessage(chatText, channel)
	end
	if playercount == 0 then
		self:ShowMessage(L["No dkp scores found matching chosen criteria"], channel)
	end
end

function EasyDKP:ShowChanges()
	if self.db.profile.changes.count == 0 then
		self:Print("no changes recorded")
	else
		for index = 1, self.db.profile.changes.count do
			self:Print(index..": "..self.db.profile.changes[index].name.." altered by "..self.db.profile.changes[index].value)
		end
	end
end

function EasyDKP:ShowMessage(chatText, channel)
	if channel == "SELF" then
		self:Print(chatText)
	elseif channel == "GUILD" then
		if IsInGuild() then
			SendChatMessage(chatText, channel)
		else
			self:Print(chatText)
		end
	elseif channel =="OFFICER" then
		if IsInGuild() then -- add and some check if can speak in officer chat
			SendChatMessage(chatText, channel)
		else
			self:Print(chatText)
		end
	elseif channel == "PARTY" then
		if UnitInParty("player") or UnitInRaid("player") then
			SendChatMessage(chatText, channel)
		else
			self:Print(chatText)
		end
	elseif channel == "RAID" then
		if UnitInRaid("player") then
			SendChatMessage(chatText, channel)
		else
			self:Print(chatText)
		end
	end
end

function EasyDKP:CheckAddSubtract(info, ...)
	-- validate command line pattern
	local cmd = info.input
	local name, value = strmatch(cmd, "add (%w+) (%d+)")
	if name and value then
		return self:AddDKP(name, value)
	end
	name, value = strmatch(cmd, "subtract (%w+) (%d+)")
	if name and value then 
		return self:SubtractDKP(name, value)
	end
	return false
end

function EasyDKP:AddDKP(name, value)
	-- returns true if succeded false otherwise
	self:Print("adding "..value.." to "..name)
	-- need checks to see that name is valid
	self:ChangeItem(name, value)
	return true
end

function EasyDKP:SubtractDKP(name, value)
	-- returns true if succeded false otherwise
	self:Print("subtracting "..value.." from "..name)
	-- need checks to see that name is valid
	self:ChangeItem(name, -value)
	return true
end

function EasyDKP:PopupQuestion(name, value)
    local sQuestion = TEXT(format(L["Do you want to subtract %s from %s ?"], value, name))
    -- subtract dialog
    StaticPopupDialogs["POPUP_DIALOG"] = {
		text = sQuestion,
		button1 = L["Yes"],
		button2 = L["No"],
		OnAccept = function()
			EasyDKP:ChangeItem(name, value * -1)
		end,
		OnCancel = function(_, reason)
--~ 			EasyDKP:CancelSubtract(name, value, reason)
		end,
		sound = "levelup2",
		timeout = 0,
		whileDead = 1,
		hideOnEscape = 1,
    }
    StaticPopup_Show ("POPUP_DIALOG")
end

function EasyDKP:ChangeItem(name, value)
	if not self.db.profile.changes then
		self:InitialiseChanges()
	end
	local index = self.db.profile.changes.count + 1
	self.db.profile.changes.count = index
	self.db.profile.changes[index] = {}
	self.db.profile.changes[index].index = index
	self.db.profile.changes[index].name = name
	self.db.profile.changes[index].value = value
	if self.db.profile.debug then
		if value > 0 then 
			self:Print(format(L["add %s %s"], name, value))
		else
			self:Print(format(L["subtract %s %s"], name, value))
		end
	end
end

-----------------------------
--  Helper functions
-----------------------------

function sortDKP(o1, o2)
    return tonumber(o1["dkp"]) > tonumber(o2["dkp"])
end

function sortChanges(o1, o2)
	if o1["name"] ~= o2["name"] then 
		return o1["name"] > o2["name"]
	else
		return o1["index"] > o2["index"]
	end
end

function EasyDKP:IncludeClass(classname)
	local ticked = 0
	for i = 1, 10 do
		if self.class[i].ticked then
			if strmatch(classname,self.class[i].name) then
				return true
			end
			ticked = ticked + 1
		end
	end
	if ticked == 0 then
		return true -- return true if no classes ticked
	else
		return false
	end
end

function EasyDKP:ClassColour(class)
	for i = 1, 10 do
		if self.class[i].name == class then
			return self.class[i].link
		end
	end
	return "|cff808080"
end

-- process values for "specials"
function EasyDKP:GetValue(itemid)
	local value = 0
	local cost = 0
	itemid = tonumber(itemid)
	if self.db.profile.debug then
		self:Print("get value of "..itemid)
	end
	local fixedValue = self:GetFixedValue(itemid)
	if fixedValue then 
		value = fixedValue
	else
		local sName, sLink, iRarity, iLevel, iMinLevel, sType, sSubType, iStackCount = GetItemInfo(itemid)
		value = iLevel
	end
	if self.db.profile.doublecost then 
		if GetNumRaidMembers() > 15 then
			cost = value * 2
			value = cost -- for 25 man raiding use full cost for value
		else
			cost = value * 1.5
		end
	else
		cost = value
	end
	return value, cost  -- value is the itemValue adjusted for specials and what members earn, cost is the DKP cost to the player
end

function EasyDKP:GetItemInfoFromLink(link)
	local _, _, itemid, enchant, _, subid, itemname = string.find(link, "|Hitem:(%d+):")
	return tonumber(itemid or 0), tonumber(enchant or 0), tonumber(subid or 0), tonumber(itemname or 0)
end

function EasyDKP:ReturnStatus()
    local ret = L["Running: "]
    if self.db.profile.enabled then
	    ret = ret.."|c0000FF00"..L["Yes"].."|r"
    else
	    ret = ret.."|c00FF0000"..L["No"].."|r"
    end
    ret = ret.."\n"..L["Debug: "]
    if self.db.profile.debug then
	    ret = ret.."|c0000FF00"..L["Yes"].."|r"
    else
	    ret = ret.."|c00FF0000"..L["No"].."|r"
    end
    local range =  self.quality[self.db.profile.minquality].link..self.quality[self.db.profile.minquality].text.."|r to "..self.quality[self.db.profile.maxquality].link..self.quality[self.db.profile.maxquality].text
    ret = ret.."\n"..L["Range: "]..range
    return ret
end
